@@ -17,6 +17,7 @@
 #include <libxml/hash.h>
 #include <libxml/xpath.h>
 #include <libxml/xmlerror.h>
+#include <libxml/dict.h>
 #include <libxslt/xslt.h>
 #include "xsltexports.h"
 #include "numbersInternals.h"
@@ -252,38 +253,38 @@ struct _xsltStylePreComp {
      * Pre computed values.
      */
 
-    xmlChar *stype;             /* sort */
+    const xmlChar *stype;       /* sort */
     int      has_stype;		/* sort */
     int      number;		/* sort */
-    xmlChar *order;             /* sort */
+    const xmlChar *order;	/* sort */
     int      has_order;		/* sort */
     int      descending;	/* sort */
-    xmlChar *lang;		/* sort */
+    const xmlChar *lang;	/* sort */
     int      has_lang;		/* sort */
-    xmlChar *case_order;	/* sort */
+    const xmlChar *case_order;	/* sort */
     int      lower_first;	/* sort */
 
-    xmlChar *use;		/* copy, element */
+    const xmlChar *use;		/* copy, element */
     int      has_use;		/* copy, element */
 
     int      noescape;		/* text */
 
-    xmlChar *name;		/* element, attribute, pi */
+    const xmlChar *name;	/* element, attribute, pi */
     int      has_name;		/* element, attribute, pi */
-    xmlChar *ns;		/* element */
+    const xmlChar *ns;		/* element */
     int      has_ns;		/* element */
 
-    xmlChar *mode;		/* apply-templates */
-    xmlChar *modeURI;		/* apply-templates */
+    const xmlChar *mode;	/* apply-templates */
+    const xmlChar *modeURI;	/* apply-templates */
 
-    xmlChar *test;		/* if */
+    const xmlChar *test;	/* if */
 
     xsltTemplatePtr templ;	/* call-template */
 
-    xmlChar *select;		/* sort, copy-of, value-of, apply-templates */
+    const xmlChar *select;	/* sort, copy-of, value-of, apply-templates */
 
     int      ver11;		/* document */
-    xmlChar *filename;		/* document URL */
+    const xmlChar *filename;	/* document URL */
     int      has_filename;	/* document */
 
     xsltNumberData numdata;	/* number */
@@ -303,12 +304,12 @@ typedef xsltStackElem *xsltStackElemPtr;
 struct _xsltStackElem {
     struct _xsltStackElem *next;/* chained list */
     xsltStylePreCompPtr comp;   /* the compiled form */
-    int computed;	/* was the evaluation done */
-    xmlChar *name;	/* the local part of the name QName */
-    xmlChar *nameURI;	/* the URI part of the name QName */
-    xmlChar *select;	/* the eval string */
-    xmlNodePtr tree;	/* the tree if no eval string or the location */
-    xmlXPathObjectPtr value; /* The value if computed */
+    int computed;		/* was the evaluation done */
+    const xmlChar *name;	/* the local part of the name QName */
+    const xmlChar *nameURI;	/* the URI part of the name QName */
+    const xmlChar *select;	/* the eval string */
+    xmlNodePtr tree;		/* the tree if no eval string or the location */
+    xmlXPathObjectPtr value;	/* The value if computed */
 };
 
 /*
@@ -421,6 +422,15 @@ struct _xsltStylesheet {
      * For keeping track of nested includes
      */
     xsltDocumentPtr includes;	/* points to last nested include */
+
+    /*
+     * dictionnary: shared between stylesheet, context and documents.
+     */
+    xmlDictPtr dict;
+    /*
+     * precompiled attribute value templates.
+     */
+    void *attVTs;
 };
 
 /*
@@ -527,6 +537,11 @@ struct _xsltTransformContext {
     unsigned long* traceCode;		/* pointer to the variable holding the mask */
 
     int parserOptions;			/* parser options xmlParserOption */
+
+    /*
+     * dictionnary: shared between stylesheet, context and documents.
+     */
+    xmlDictPtr dict;
 };
 
 /**
@@ -616,6 +631,18 @@ XSLTPUBFUN int XSLTCALL
 XSLTPUBFUN void XSLTCALL			
 			xsltFreeRVTs		(xsltTransformContextPtr ctxt);
 			
+/*
+ * Extra functions for Attribute Value Templates
+ */
+XSLTPUBFUN void XSLTCALL
+			xsltCompileAttr		(xsltStylesheetPtr style,
+						 xmlAttrPtr attr);
+XSLTPUBFUN xmlChar * XSLTCALL
+			xsltEvalAVT		(xsltTransformContextPtr ctxt,
+						 void *avt,
+						 xmlNodePtr node);
+XSLTPUBFUN void XSLTCALL
+			xsltFreeAVTList		(void *avt);
 #ifdef __cplusplus
 }
 #endif
